﻿<%@ WebHandler Language="C#" Class="upload" %>

using System;
using System.Web;
using System.Drawing;

public class upload : IHttpHandler
{
    public void ProcessRequest(HttpContext context)
    {
        // 这里可以添加上传权限控制

        context.Response.ContentType = "text/plain";

        // 获得浏览器端 传过来 第一个文件选择框的数据  
        HttpPostedFile hpFile = context.Request.Files["Filedata"];

        // 要保存的目录路径  
        string uploadpath = GetConfig("uploadpath");

        // 判断上传文件数据的长度是否>0  
        if (hpFile != null && hpFile.ContentLength > 0)
        {
            if (hpFile.ContentLength > Convert.ToInt32(GetConfig("sizeLimit").Replace("MB", "")) * 1024 * 1024)
            {
                context.Response.StatusCode = 500;
                context.Response.Write("文件过大");
                return;
            }

            // 新生成一个文件名  
            string fileName = Guid.NewGuid().ToString().Replace("-", "");

            // 获取文件后缀名
            string fileExt = System.IO.Path.GetExtension(hpFile.FileName);

            bool isAllowType = false;
            string allowType = GetConfig("allowType");
            string[] allowTypes = allowType.Split(';');
            foreach (string ext in allowTypes)
            {
                if (ext.Replace("*", "") == fileExt)
                {
                    isAllowType = true;
                    break;
                }
            }

            if (!isAllowType)
            {
                context.Response.StatusCode = 500;
                context.Response.Write("文件格式不允许");
                return;
            }

            // 获得要保存的物理路径  
            string filePath = uploadpath + fileName + fileExt;
            string physical_FilePath = context.Server.MapPath("~" + filePath);

            // 将上传来的文件数据保存在对应的物理路径上  
            hpFile.SaveAs(physical_FilePath);

            // 生成缩略图
            string thubFilePath = uploadpath + fileName + "_thub" + fileExt;
            string physical_ThubFilePath = context.Server.MapPath("~" + thubFilePath);

            MakeThumNail(physical_FilePath, physical_ThubFilePath, 75, 50, "Cut");

            context.Response.Write(filePath + "|" + thubFilePath);
        }
        else
        {
            context.Response.Write("文件不存在");
        }
    }

    public bool IsReusable
    {
        get
        {
            return false;
        }
    }

    private string GetConfig(string key)
    {
        return System.Web.Configuration.WebConfigurationManager.AppSettings[key];
    }

    /// <summary>
    /// 生成缩略图
    /// </summary>
    /// <param name="originalImagePath">原图片地址</param>
    /// <param name="thumNailPath">缩略图地址</param>
    /// <param name="width">缩略图宽度</param>
    /// <param name="height">缩略图高度</param>
    /// <param name="model">生成缩略的模式</param>
    private void MakeThumNail(string originalImagePath, string thumNailPath, int width, int height, string model)
    {
        MakeThumNail(originalImagePath, thumNailPath, width, height, model, System.Drawing.Drawing2D.SmoothingMode.Default, System.Drawing.Drawing2D.InterpolationMode.Default);
    }

    /// <summary>
    /// 生成缩略图
    /// </summary>
    /// <param name="originalImagePath">原图片地址</param>
    /// <param name="thumNailPath">缩略图地址</param>
    /// <param name="width">缩略图宽度</param>
    /// <param name="height">缩略图高度</param>
    /// <param name="model">生成缩略的模式</param>
    /// <param name="smoothingMode">平滑程度</param>
    /// <param name="interpolationMode">查值法</param>
    private void MakeThumNail(string originalImagePath, string thumNailPath, int width, int height, string model, System.Drawing.Drawing2D.SmoothingMode smoothingMode, System.Drawing.Drawing2D.InterpolationMode interpolationMode)
    {
        System.Drawing.Image originalImage = System.Drawing.Image.FromFile(originalImagePath);

        int thumWidth = width;      //缩略图的宽度
        int thumHeight = height;    //缩略图的高度

        int x = 0;
        int y = 0;

        int originalWidth = originalImage.Width;    //原始图片的宽度
        int originalHeight = originalImage.Height;  //原始图片的高度

        switch (model)
        {
            case "HW":      //指定高宽缩放,可能变形
                break;
            case "W":       //指定宽度,高度按照比例缩放
                if (width < originalImage.Width)
                {
                    thumHeight = originalImage.Height * width / originalImage.Width;
                }
                else
                {
                    thumWidth = originalWidth;
                    thumHeight = originalHeight;
                }
                break;
            case "H":       //指定高度,宽度按照等比例缩放
                thumWidth = originalImage.Width * height / originalImage.Height;
                break;
            case "Cut":
                if ((double)originalImage.Width / (double)originalImage.Height > (double)thumWidth / (double)thumHeight)
                {
                    originalHeight = originalImage.Height;
                    originalWidth = originalImage.Height * thumWidth / thumHeight;
                    y = 0;
                    x = (originalImage.Width - originalWidth) / 2;
                }
                else
                {
                    originalWidth = originalImage.Width;
                    originalHeight = originalWidth * height / thumWidth;
                    x = 0;
                    y = (originalImage.Height - originalHeight) / 2;
                }
                break;
            default:
                break;
        }

        //新建一个bmp图片
        System.Drawing.Image bitmap = new System.Drawing.Bitmap(thumWidth, thumHeight);

        //新建一个画板
        System.Drawing.Graphics graphic = System.Drawing.Graphics.FromImage(bitmap);

        //设置高质量查值法
        graphic.InterpolationMode = interpolationMode;

        //设置高质量，低速度呈现平滑程度
        graphic.SmoothingMode = smoothingMode;

        //清空画布并以透明背景色填充
        graphic.Clear(System.Drawing.Color.Transparent);

        //在指定位置并且按指定大小绘制原图片的指定部分
        graphic.DrawImage(originalImage, new System.Drawing.Rectangle(0, 0, thumWidth, thumHeight), new System.Drawing.Rectangle(x, y, originalWidth, originalHeight), System.Drawing.GraphicsUnit.Pixel);

        try
        {
            bitmap.Save(thumNailPath);
        }
        catch (Exception ex)
        {
            throw ex;
        }
        finally
        {
            originalImage.Dispose();
            bitmap.Dispose();
            graphic.Dispose();
        }
    }
}